@@ -31,20 +31,18 @@
 #define DESCRIPTION_LENGTH 256
 
 static libspectrum_error
-write_rom( libspectrum_tape_block *block, libspectrum_byte **buffer,
-	   libspectrum_byte **ptr, size_t *length, libspectrum_id_t type );
+write_rom( libspectrum_tape_block *block, libspectrum_buffer *buffer,
+	   libspectrum_id_t type );
 static libspectrum_error
-write_turbo( libspectrum_tape_block *block, libspectrum_byte **buffer,
-	     libspectrum_byte **ptr, size_t *length, libspectrum_id_t type );
+write_turbo( libspectrum_tape_block *block, libspectrum_buffer *buffer,
+             libspectrum_id_t type );
 static libspectrum_error
-write_pure_data( libspectrum_tape_block *block, libspectrum_byte **buffer,
-		 libspectrum_byte **ptr, size_t *length,
-		 libspectrum_id_t type );
+write_pure_data( libspectrum_tape_block *block, libspectrum_buffer *buffer,
+                 libspectrum_id_t type );
 
 static libspectrum_error
-write_tap_block( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		 size_t *length, libspectrum_byte *data, size_t data_length,
-		 libspectrum_id_t type );
+write_tap_block( libspectrum_buffer *buffer, libspectrum_byte *data,
+                 size_t data_length, libspectrum_id_t type );
 static libspectrum_error
 skip_block( libspectrum_tape_block *block, const char *message );
 
@@ -133,15 +131,13 @@ internal_tap_read( libspectrum_tape *tape, const libspectrum_byte *buffer,
 }
 
 libspectrum_error
-internal_tap_write( libspectrum_byte **buffer, size_t *length,
-		    libspectrum_tape *tape, libspectrum_id_t type )
+internal_tap_write( libspectrum_buffer *buffer, libspectrum_tape *tape,
+                    libspectrum_id_t type )
 {
   libspectrum_tape_iterator iterator;
   libspectrum_tape_block *block;
   libspectrum_error error;
 
-  libspectrum_byte *ptr = *buffer;
-
   for( block = libspectrum_tape_iterator_init( &iterator, tape );
        block;
        block = libspectrum_tape_iterator_next( &iterator ) )
@@ -151,20 +147,20 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     switch( libspectrum_tape_block_type( block ) ) {
 
     case LIBSPECTRUM_TAPE_BLOCK_ROM:
-      error = write_rom( block, buffer, &ptr, length, type );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return error; }
+      error = write_rom( block, buffer, type );
+      if( error != LIBSPECTRUM_ERROR_NONE ) { return error; }
       done = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_TURBO:
-      error = write_turbo( block, buffer, &ptr, length, type );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return error; }
+      error = write_turbo( block, buffer, type );
+      if( error != LIBSPECTRUM_ERROR_NONE ) { return error; }
       done = 1;
       break;
 
     case LIBSPECTRUM_TAPE_BLOCK_PURE_DATA:
-      error = write_pure_data( block, buffer, &ptr, length, type );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return error; }
+      error = write_pure_data( block, buffer, type );
+      if( error != LIBSPECTRUM_ERROR_NONE ) { return error; }
       done = 1;
       break;
 
@@ -178,7 +174,7 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_PULSE_SEQUENCE:
     case LIBSPECTRUM_TAPE_BLOCK_DATA_BLOCK:
       error = skip_block( block, "conversion almost certainly won't work" );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { return 1; }
       done = 1;
       break;
 
@@ -187,7 +183,7 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_SELECT:
     case LIBSPECTRUM_TAPE_BLOCK_SET_SIGNAL_LEVEL:
       error = skip_block( block, "conversion may not work" );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { return 1; }
       done = 1;
       break;
 
@@ -201,13 +197,12 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
     case LIBSPECTRUM_TAPE_BLOCK_CUSTOM:
     case LIBSPECTRUM_TAPE_BLOCK_CONCAT:
       error = skip_block( block, NULL );
-      if( error != LIBSPECTRUM_ERROR_NONE ) { libspectrum_free( *buffer ); return 1; }
+      if( error != LIBSPECTRUM_ERROR_NONE ) { return 1; }
       done = 1;
       break;
     }
 
     if( !done ) {
-      if( *length ) libspectrum_free( *buffer );
       libspectrum_print_error(
         LIBSPECTRUM_ERROR_LOGIC,
         "libspectrum_tap_write: unknown block type 0x%02x",
@@ -218,20 +213,16 @@ internal_tap_write( libspectrum_byte **buffer, size_t *length,
 
   }
 
-
-  (*length) = ptr - *buffer;
-
   return LIBSPECTRUM_ERROR_NONE;
 }
 
 static libspectrum_error
-write_rom( libspectrum_tape_block *block, libspectrum_byte **buffer,
-	   libspectrum_byte **ptr, size_t *length, libspectrum_id_t type )
+write_rom( libspectrum_tape_block *block, libspectrum_buffer *buffer,
+	   libspectrum_id_t type )
 {
   libspectrum_error error;
 
-  error = write_tap_block( buffer, ptr, length,
-			   libspectrum_tape_block_data( block ),
+  error = write_tap_block( buffer, libspectrum_tape_block_data( block ),
 			   libspectrum_tape_block_data_length( block ), type );
   if( error != LIBSPECTRUM_ERROR_NONE ) return error;
 
@@ -239,8 +230,8 @@ write_rom( libspectrum_tape_block *block, libspectrum_byte **buffer,
 }
 
 static libspectrum_error
-write_turbo( libspectrum_tape_block *block, libspectrum_byte **buffer,
-	     libspectrum_byte **ptr, size_t *length, libspectrum_id_t type )
+write_turbo( libspectrum_tape_block *block, libspectrum_buffer *buffer,
+             libspectrum_id_t type )
 {
   libspectrum_error error;
 
@@ -250,8 +241,7 @@ write_turbo( libspectrum_tape_block *block, libspectrum_byte **buffer,
     "write_turbo: converting Turbo Speed Data Block (ID 0x11); conversion may well not work"
   );
 
-  error = write_tap_block( buffer, ptr, length,
-			   libspectrum_tape_block_data( block ),
+  error = write_tap_block( buffer, libspectrum_tape_block_data( block ),
 			   libspectrum_tape_block_data_length( block ), type );
   if( error != LIBSPECTRUM_ERROR_NONE ) return error;
 
@@ -259,8 +249,7 @@ write_turbo( libspectrum_tape_block *block, libspectrum_byte **buffer,
 }
 
 static libspectrum_error
-write_pure_data( libspectrum_tape_block *block, libspectrum_byte **buffer,
-		 libspectrum_byte **ptr, size_t *length,
+write_pure_data( libspectrum_tape_block *block, libspectrum_buffer *buffer,
 		 libspectrum_id_t type )
 {
   libspectrum_error error;
@@ -271,8 +260,7 @@ write_pure_data( libspectrum_tape_block *block, libspectrum_byte **buffer,
     "write_pure_data: converting Pure Data Block (ID 0x14); conversion almost certainly won't work"
   );
 
-  error = write_tap_block( buffer, ptr, length,
-			   libspectrum_tape_block_data( block ),
+  error = write_tap_block( buffer, libspectrum_tape_block_data( block ),
 			   libspectrum_tape_block_data_length( block ), type );
   if( error != LIBSPECTRUM_ERROR_NONE ) return error;
 
@@ -280,11 +268,11 @@ write_pure_data( libspectrum_tape_block *block, libspectrum_byte **buffer,
 }
 
 static libspectrum_error
-write_tap_block( libspectrum_byte **buffer, libspectrum_byte **ptr,
-		 size_t *length, libspectrum_byte *data, size_t data_length,
-		 libspectrum_id_t type )
+write_tap_block( libspectrum_buffer* buffer, libspectrum_byte *data,
+                 size_t data_length, libspectrum_id_t type )
 {
   size_t buf_length;
+  libspectrum_byte parity;
 
   /* Discard the parity byte for STA files */
   if( type == LIBSPECTRUM_ID_TAPE_STA )
@@ -303,18 +291,16 @@ write_tap_block( libspectrum_byte **buffer, libspectrum_byte **ptr,
     data_length -= 2;
   }
 
-  libspectrum_make_room( buffer, 2 + buf_length, ptr, length );
-
   /* Write out the length and the data */
-  *(*ptr)++ =   data_length & 0x00ff;
-  *(*ptr)++ = ( data_length & 0xff00 ) >> 8;
-  memcpy( *ptr, data, buf_length );
+  libspectrum_buffer_write_byte( buffer, data_length & 0x00ff );
+  libspectrum_buffer_write_byte( buffer, ( data_length & 0xff00 ) >> 8 );
+  libspectrum_buffer_write( buffer, data, buf_length - 1 );
   
   /* Fix the parity for SPC files */
+  parity = data[ buf_length - 1];
   if( type == LIBSPECTRUM_ID_TAPE_SPC )
-    (*ptr)[ buf_length - 1 ] ^= (*ptr)[0];
-
-  (*ptr) += buf_length;
+    parity ^= data[0];
+  libspectrum_buffer_write_byte( buffer, parity );
 
   return LIBSPECTRUM_ERROR_NONE;
 }
