@@ -1,3 +1,5 @@
+#include <config.h>
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -798,6 +800,46 @@ static struct test_description tests[] = {
   { test_28, "Zero tail length PZX file", 0 },
   { test_29, "No pilot pulse GDB TZX file", 0 },
   { test_30, "CSW conversion", 0 },
+  { test_31, "Write SZX Z80R chunk", 0 },
+  { test_32, "Write SZX SPCR chunk", 0 },
+  { test_33, "Write SZX JOY chunk", 0 },
+  { test_34, "Write SZX KEYB chunk", 0 },
+  { test_35, "Write SZX ZXPR chunk", 0 },
+  { test_36, "Write SZX AY chunk", 0 },
+  { test_37, "Write SZX SCLD chunk", 0 },
+  { test_38, "Write SZX ZXAT chunk", 0 },
+  { test_39, "Write SZX ZXCF chunk", 0 },
+  { test_40, "Write SZX AMXM chunk", 0 },
+  { test_41, "Write SZX SIDE chunk", 0 },
+  { test_42, "Write SZX DRUM chunk", 0 },
+  { test_43, "Write SZX COVX chunk", 0 },
+  { test_44, "Read SZX Z80R chunk", 0 },
+  { test_45, "Read SZX SPCR chunk", 0 },
+  { test_46, "Read SZX JOY chunk", 0 },
+  { test_47, "Read SZX KEYB chunk", 0 },
+  { test_48, "Read SZX ZXPR chunk", 0 },
+  { test_49, "Read SZX AY chunk", 0 },
+  { test_50, "Read SZX SCLD chunk", 0 },
+  { test_51, "Read SZX ZXAT chunk", 0 },
+  { test_52, "Read SZX ZXCF chunk", 0 },
+  { test_53, "Read SZX AMXM chunk", 0 },
+  { test_54, "Read SZX SIDE chunk", 0 },
+  { test_55, "Read SZX DRUM chunk", 0 },
+  { test_56, "Read SZX COVX chunk", 0 },
+  { test_57, "Write SZX ZMMC chunk", 0 },
+  { test_58, "Read SZX ZMMC chunk", 0 },
+  { test_59, "Write SZX RAMP chunk", 0 },
+  { test_60, "Read SZX RAMP chunk", 0 },
+  { test_61, "Write SZX ATRP chunk", 0 },
+  { test_62, "Write SZX CFRP chunk", 0 },
+  { test_63, "Read SZX ATRP chunk", 0 },
+  { test_64, "Read SZX CFRP chunk", 0 },
+  { test_65, "Write uncompressed SZX RAMP chunk", 0 },
+  { test_66, "Write uncompressed SZX ATRP chunk", 0 },
+  { test_67, "Write uncompressed SZX CFRP chunk", 0 },
+  { test_68, "Read uncompressed SZX RAMP chunk", 0 },
+  { test_69, "Read uncompressed SZX ATRP chunk", 0 },
+  { test_70, "Read uncompressed SZX CFRP chunk", 0 }
 };
 
 static size_t test_count = ARRAY_SIZE( tests );
