@@ -296,6 +296,7 @@ tzx_read_turbo_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
 {
   libspectrum_tape_block* block;
   size_t length; libspectrum_byte *data;
+  libspectrum_byte bits_in_last_byte;
   libspectrum_error error;
 
   /* Check there's enough left in the buffer for all the metadata */
@@ -328,7 +329,10 @@ tzx_read_turbo_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
   libspectrum_tape_block_set_pilot_pulses( block,
 					   (*ptr)[0] + (*ptr)[1] * 0x100 );
   (*ptr) += 2;
-  libspectrum_tape_block_set_bits_in_last_byte( block, **ptr ); (*ptr)++;
+
+  bits_in_last_byte = **ptr;
+  (*ptr)++;
+
   libspectrum_set_pause_ms               ( block,
 					   (*ptr)[0] + (*ptr)[1] * 0x100 );
   (*ptr) += 2;
@@ -336,6 +340,18 @@ tzx_read_turbo_block( libspectrum_tape *tape, const libspectrum_byte **ptr,
   /* Read the data in */
   error = tzx_read_data( ptr, end, &length, 3, &data );
   if( error ) { libspectrum_free( block ); return error; }
+
+  if( bits_in_last_byte == 0 && length >= 1 ) {
+    bits_in_last_byte = 8;
+    length -= 1;
+  }
+
+  if( bits_in_last_byte > 8 ) {
+    bits_in_last_byte = 8;
+  }
+
+  libspectrum_tape_block_set_bits_in_last_byte( block, bits_in_last_byte );
+
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
@@ -423,7 +439,7 @@ tzx_read_pure_data( libspectrum_tape *tape, const libspectrum_byte **ptr,
 {
   libspectrum_tape_block* block;
   size_t length; libspectrum_byte *data;
-
+  libspectrum_byte bits_in_last_byte;
   libspectrum_error error;
 
   /* Check there's enough left in the buffer for all the metadata */
@@ -444,13 +460,27 @@ tzx_read_pure_data( libspectrum_tape *tape, const libspectrum_byte **ptr,
   libspectrum_tape_block_set_bit1_length( block,
 					  (*ptr)[0] + (*ptr)[1] * 0x100 );
   (*ptr) += 2;
-  libspectrum_tape_block_set_bits_in_last_byte( block, **ptr ); (*ptr)++;
+
+  bits_in_last_byte = **ptr;
+  (*ptr)++;
+
   libspectrum_set_pause_ms( block, (*ptr)[0] + (*ptr)[1] * 0x100 );
   (*ptr) += 2;
 
   /* And the actual data */
   error = tzx_read_data( ptr, end, &length, 3, &data );
   if( error ) { libspectrum_free( block ); return error; }
+
+  if( bits_in_last_byte == 0 && length > 1 ) {
+    bits_in_last_byte = 8;
+    length -= 1;
+  }
+
+  if( bits_in_last_byte > 8 ) {
+    bits_in_last_byte = 8;
+  }
+
+  libspectrum_tape_block_set_bits_in_last_byte( block, bits_in_last_byte );
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
@@ -465,6 +495,7 @@ tzx_read_raw_data (libspectrum_tape *tape, const libspectrum_byte **ptr,
 {
   libspectrum_tape_block* block;
   size_t length; libspectrum_byte *data;
+  libspectrum_byte bits_in_last_byte;
   libspectrum_error error;
 
   /* Check there's enough left in the buffer for all the metadata */
@@ -480,12 +511,24 @@ tzx_read_raw_data (libspectrum_tape *tape, const libspectrum_byte **ptr,
   libspectrum_tape_block_set_bit_length( block,
 					 (*ptr)[0] + (*ptr)[1] * 0x100 );
   libspectrum_set_pause_ms( block, (*ptr)[2] + (*ptr)[3] * 0x100 );
-  libspectrum_tape_block_set_bits_in_last_byte( block, (*ptr)[4] );
+
+  bits_in_last_byte = (*ptr)[4];
   (*ptr) += 5;
 
   /* And the actual data */
   error = tzx_read_data( ptr, end, &length, 3, &data );
   if( error ) { libspectrum_free( block ); return error; }
+
+  if( bits_in_last_byte == 0 && length >= 1 ) {
+    bits_in_last_byte = 8;
+    length -= 1;
+  }
+
+  if( bits_in_last_byte > 8 ) {
+    bits_in_last_byte = 8;
+  }
+
+  libspectrum_tape_block_set_bits_in_last_byte( block, bits_in_last_byte );
   libspectrum_tape_block_set_data_length( block, length );
   libspectrum_tape_block_set_data( block, data );
 
